#ifndef _ResourceGroup_Loader_hpp__
#define _ResourceGroup_Loader_hpp__

#include <OgrePrerequisites.h>
#include <OgreWorkQueue.h>
#include <OgreStringVector.h>
#include <OgreSingleton.h>
#include <OgreFrameListener.h>

class ResourceGroupBackgroundLoader : public Ogre::Singleton<ResourceGroupBackgroundLoader>, public Ogre::WorkQueue::RequestHandler, public Ogre::WorkQueue::ResponseHandler, public Ogre::FrameListener
{
public:
	static ResourceGroupBackgroundLoader& getSingleton();
	static ResourceGroupBackgroundLoader* getSingletonPtr();

	struct Listener
	{
		virtual void resGroupCreated(const Ogre::String& resGroupName){}
		virtual void resGroupLoaded(const Ogre::String& resGroupName){}
		virtual void resGroupUnLoaded(const Ogre::String& resGroupName){}
		virtual void meshLoaded(const Ogre::String& meshName){}
		virtual void meshUnLoaded(const Ogre::String& meshName){}
		virtual ~Listener() {}
	};

	ResourceGroupBackgroundLoader(DWORD loadingIntervalMs);
	~ResourceGroupBackgroundLoader();

	void loadResourceGroup(const Ogre::String& resGroupName, const Ogre::String& location, Listener* listener = 0);
	void unloadResourceGroup(const Ogre::String& resGroupName, Listener* listener = 0);

private:
	bool frameEnded(const Ogre::FrameEvent &evt);

private:
	enum RequestType
	{
		LOAD_GROUP,
		UNLOAD_GROUP,
	};

	enum ResGroupStatus
	{
		INITIALISE,
		PREPARING,
		PREPARED,
		COMPLETED
	};

	struct ResGroupRequest
	{
		RequestType					type;
		ResGroupStatus				status;
		Ogre::String				name;
		Ogre::String				location;
		Listener*					listener;
		Ogre::StringVectorPtr		meshes;

		typedef Ogre::deque<Ogre::ResourcePtr>::type ResourceQueue;
		ResourceQueue				resourceQueue;
	};
	typedef Ogre::deque<ResGroupRequest>::type ResGroupRequestQueue;

	DWORD							mLoadingIntervalMs;
	ResGroupRequestQueue			mResGroupRequestQueue;

	Ogre::uint16					mWorkQueueChannel;

	struct Request
	{
		RequestType				type;
		Ogre::String			resGroupName;
		Ogre::String			resName;

		friend std::ostream& operator << (std::ostream& o, const Request& r)
		{
			(void)r; return o;
		}
	};

	bool canHandleRequest(const Ogre::WorkQueue::Request* req, const Ogre::WorkQueue* srcQ) { return true; }
	Ogre::WorkQueue::Response* handleRequest(const Ogre::WorkQueue::Request* req, const Ogre::WorkQueue* srcQ);
	bool canHandleResponse(const Ogre::WorkQueue::Response* res, const Ogre::WorkQueue* srcQ) { return true; }
	void handleResponse(const Ogre::WorkQueue::Response* res, const Ogre::WorkQueue* srcQ);

	void getResourcesFromResourceGroup(bool load);
};

#endif
